#!/bin/bash
#echo PARAMETER: $*
PROGPATH=/share/microchip/PK2CMD
PROG=pk2cmd
MYSELF="$0"
OLDPATH=$PATH
PATH=$PATH:$PROGPATH

help()
{
  clear
  echo
  echo "Call script as followed:"
  echo
  echo "$MYSELF HEXNUMBER|AUTO -f ProjectDir ImagePath  -PDevicename  -A{Voltage} -E -M -J1 -K"
  echo
  echo "{Voltage} is the voltage (doted) value: example 4.5"
  echo "HEXNUMBER is the last address of program : allowed chars : 0 - 9 A - F a - f" 
  echo
  echo "help for $PROG and used parameters:"
  echo
  $PROGPATH/$PROG ${PARAM1}${HEXFILE} "?"
  
  exit 0
}

MEMMAX="$1"
shift

if [[ $MEMMAX == *"help"* ]]
then
  help
fi

 
echo
echo "--------------------------------------"
echo "| Pickit2 wrapper (c) H. Rueter 2023 |"
echo "--------------------------------------"
echo
echo using commandline : $*
echo
echo Shift, verify and assign parameters for call off "$PROG"
echo


PARAM1="$1"
shift
PARAM2="$1"
shift
PARAM3="$1"
shift

PICMODEL="$1"
shift
PICMODEL=`echo $PICMODEL|tr '[:lower:]' '[:upper:]'|sed 's/-P//'`

VOLTAGE="$1"
shift

VOLTAGE=`echo $VOLTAGE|tr '[:lower:]' '[:upper:]'|sed 's/-A//'`


HEXFILE="$PARAM2/$PARAM3"
MEMMAXDEC=""
MEMMAX=`echo "$MEMMAX" | tr '[:lower:]' '[:upper:]'`
DIRNAME=`dirname "$HEXFILE"`/memoryfile.xml

if [ "a$MEMMAX" =  "aAUTO" ]
then
  DIRNAME=`dirname "$HEXFILE"`/memoryfile.xml
  MEMMAXDEC=`expr \`xmllint --xpath 'string(/project/executable/memory [@name="program"]/length)' $DIRNAME \`   - 1`
  MEMMAX=`printf '0x%x' $MEMMAXDEC`
else
  if ! [[ $MEMMAX =~ ^[0-9A-Fa-f]{1,}$ ]]
  then
    echo "$MEMMAX is not a valid hex number "
    echo "example : valid chars are 0 .. 9 A B C D E F"
    exit 1
  fi
  MEMMAXDEC=`echo "ibase=16; $MEMMAX"|bc`
  MEMMAX="0x"$MEMMAX
fi

echo
echo "MEMMAX: $MEMMAX ($MEMMAXDEC)"
echo 
echo "---------------------"
echo "| Start Programming |"
echo "---------------------"
echo


echo "Calling: " $PROGPATH/$PROG ${PARAM1}${HEXFILE} -P$PICMODEL $*
 
$PROGPATH/$PROG ${PARAM1}${HEXFILE}  -P$PICMODEL $*


RETURNSTATUS=$? 

case  $RETURNSTATUS in
  39)
    echo "ERRORMSG: Error opening or loading a specified hexfile (-f...)."
    ;;
  37) 
    echo "ERRORMSG: A part autodetect operation failed to find a known part."
    ;;
  36) 
    echo "ERRORMSG: A syntax error in a command line argument was detected, an invalid combination of operations was entered, an invalid value was entered"
    ;;
  34) 
   echo "ERRORMSG: program or verify operation failed"
    ;;
  28) 
    echo "ERRORMSG: firmware upgrade failed"
    ;;
  24) 
    echo "ERRORMSG: The PK2CMD executable cannot find the device file PK2DeviceFile.dat or the device file"
    ;;
  15) 
    echo "ERRORMSG: A file specified for reading to (-gf...) cannot be opened for writing."
    ;;
  11) 
    echo "ERRORMSG: OS firmware must be updated before being used with this version of PK2CMD."
    ;;
  10) 
    echo "ERRORMSG: No programmer found"
    ;;
  7) 
    echo "ERRORMSG: Returned if an operation fails and a more specific error does not exist."
    ;;
  5)
    echo "ERRORMSG: Voltage error."
   ;;
  1) 
    echo "ERRORMSG: Unknown error 1"
   ;;
  0)
    echo
    echo "--------------------"
    echo "| Programming done |"
    echo "--------------------"
    echo
    echo
    echo "---------------------------"
    echo "| Verify programm hexcode |"
    echo "---------------------------"
    echo

    $PROGPATH/$PROG -p$PICMODEL  -A4.7 -GP0x000-$MEMMAX -GI -GC
    ;;
esac

if [ $RETURNSTATUS -ne 0 ]
then
  echo
  echo "----------------------"
  echo "| Programming FAILED |"
  echo "----------------------"
  help
fi

echo
echo "----------"
echo "| Finish |"
echo "----------"
echo
echo

PATH=%OLDPATH%

